<?php

/**
 * Admin settings form.
 */
function qbo_api_admin_settings($form, &$form_state) {
  module_load_install('qbo_api');

  // Extract API keys.
  extract(qbo_api_keys());

  // Requirements.
  $form['requirements'] = array(
    '#type' => 'fieldset',
    '#title' => t('Requirements'),
  );
  $form['requirements']['report'] = array(
    '#markup' => theme('status_report', array('requirements' => qbo_api_requirements('runtime'))),
  );

  // Status.
  $form['status'] = array(
    '#type' => 'fieldset',
    '#title' => t('Status'),
  );
  $form['status']['report'] = array(
    '#type' => 'html_tag',
    '#tag' => 'div',
    '#attributes' => array(
      'style' => 'font-weight: bold; color: ' . ($creds_loaded ? 'green' : 'red') . ';',
    ),
    '#value' => $creds_loaded ? t('Connected') : t('Disconnected'),
  );

  // Connection.
  $form['connection'] = array(
    '#type' => 'fieldset',
    '#title' => t('Connection'),
  );
  $form['connection']['help'] = array(
    '#type' => 'fieldset',
    '#title' => t('Help'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['connection']['help']['list'] = array(
    '#theme' => 'item_list',
    '#type' => 'ol',
    '#items' => array(
      t('Log in to your Quickbooks Online.'),
      t('Click your company name at the top-right then select "Account and Settings".'),
      t('Click on "Billing & Support".'),
      t('Copy the "Company ID" (remove spaces) at the top in to the connection settings here.'),
      t('Go to https://developer.intuit.com and either create a developer account or sign-in. If you are creating a new account, it must be through the account you use to log in to Quickbooks.'),
      t('On the top-menu, click "My apps" then "Create new app" and choose "Quickbooks API". The name of the app and the URLs do not matter.'),
      t('Click on your new app, and navigate to "QuickBooks sandbox" under the Resources section.  Add a sandbox company.'),
      t('Go back to "My apps", copy the "oAuth Consumer" and "oAuth Consumer Secret" tokens into this form and click Save.'),
      t('After saving, the oAuth connection section should be visible below. Click on the connect button to finish the connection.'),
    ),
  );
  $form['connection']['qbo_api_company_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Company ID'),
    '#required' => TRUE,
    '#default_value' => $company_id,
  );
  $form['connection']['qbo_api_consumer_key'] = array(
    '#type' => 'textfield',
    '#title' => t('Consumer key'),
    '#required' => TRUE,
    '#default_value' => $consumer_key,
  );
  $form['connection']['qbo_api_consumer_secret'] = array(
    '#type' => 'textfield',
    '#title' => t('Consumer secret'),
    '#required' => TRUE,
    '#default_value' => $consumer_secret,
  );
  $form['connection']['oauth'] = array(
    '#type' => 'fieldset',
    '#title' => t('OAuth connection'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#access' => (bool) ($company_id && $consumer_key && $consumer_secret),
  );
  $form['connection']['oauth']['button'] = array(
    '#access' => (bool) !$access_token,
    '#markup' => '<ipp:connectToIntuit></ipp:connectToIntuit>',
    '#attached' => array(
      'js' => array(
        'https://appcenter.intuit.com/Content/IA/intuit.ipp.anywhere.js' => array(
          'type' => 'external',
          'weight' => 1
        ),
        drupal_get_path('module', 'qbo_api') . '/js/qbo_api.js' => array(
          'weight' => 2,
        ),
        array(
          'type' => 'setting',
          'data' => array(
            'qboApi' => array(
              'oauthUrl' => 'admin/config/services/qbo/oauth',
            ),
          ),
        ),
      ),
    ),
  );
  $form['connection']['oauth']['reconnect'] = array(
    '#type' => 'submit',
    '#value' => t('Reconnect'),
    '#limit_validation_errors' => array(),
    '#submit' => array('qbo_api_oauth_actions_reconnect'),
    '#access' => (bool) $access_token && (REQUEST_TIME > (QBO_API_TOKEN_REFRESH + $auth_date)),
  );
  $form['connection']['oauth']['disconnect'] = array(
    '#type' => 'submit',
    '#value' => t('Disconnect'),
    '#limit_validation_errors' => array(),
    '#submit' => array('qbo_api_oauth_actions_disconnect'),
    '#access' => (bool) $access_token,
  );
  $form['connection']['oauth']['tokens'] = array(
    '#type' => 'fieldset',
    '#title' => t('Tokens'),
    '#description' => !$creds_loaded ? t('The tokens below will be automatically populated after connecting using the button above.') : '',
  );
  $form['connection']['oauth']['tokens']['qbo_api_access_token'] = array(
    '#type' => 'textfield',
    '#title' => t('Access token'),
    '#required' => FALSE,
    '#default_value' => $access_token,
  );
  $form['connection']['oauth']['tokens']['qbo_api_access_secret'] = array(
    '#type' => 'textfield',
    '#title' => t('Access token secret'),
    '#required' => FALSE,
    '#default_value' => $access_secret,
  );

  return system_settings_form($form);
}

/**
 * OAuth action disconnect callback.
 */
function qbo_api_oauth_actions_disconnect($form, &$form_state) {
  qbo_api_disconnect();
  drupal_set_message(t('Disconnected successfully.'));
}

/**
 * OAuth action reconnect callback.
 */
function qbo_api_oauth_actions_reconnect($form, &$form_state) {
  if (qbo_api_reconnect()) {
    drupal_set_message(t('Reconnected successfully.'));
  }
  else {
    drupal_set_message(t('Reconnect failed. Check your settings and try again.'), 'error');
  }
}

/**
 * Calls OAuth library to request and set Access Key and Secret tokens
 */
function qbo_api_generate_oauth_tokens() {
  $callback_url = url('admin/config/services/qbo/oauth', array('absolute' => TRUE));

  // Grab the tokens from storage.
  extract(qbo_api_keys());

  // We can't generate a token if the consumer keys aren't set.
  if (empty($consumer_key) || empty($consumer_secret)) {
    drupal_set_message(t('WARNING: OAuth Consumer Key and Consumer Secret token must be set before requesting Access tokens.'), 'error');
    return '';
  }

  try {
    $oauth = new OAuth($consumer_key, $consumer_secret, OAUTH_SIG_METHOD_HMACSHA1, OAUTH_AUTH_TYPE_URI);
    $oauth->enableDebug();
    $oauth->disableSSLChecks(); // To avoid the error: (Peer certificate cannot be authenticated with given CA certificates).

    if (!isset($_GET['oauth_token']) && empty($access_token)) {
      // step 1: get request token from Intuit.
      $request_token = $oauth->getRequestToken(QBO_API_OAUTH_URL_REQUEST, $callback_url);
      $_SESSION['qbo_api_secret'] = $request_token['oauth_token_secret'];

      // step 2: send user to intuit to authorize.
      header('Location: ' . QBO_API_OAUTH_URL_AUTHORISE . '?oauth_token=' . $request_token['oauth_token']);
    }

    if (isset($_GET['oauth_token']) && isset($_GET['oauth_verifier'])) {
      // step 3: request a access token from Intuit.
      $oauth->setToken($_GET['oauth_token'], $_SESSION['qbo_api_secret']);
      $access_token = $oauth->getAccessToken(QBO_API_OAUTH_URL_ACCESS);

      variable_set('qbo_api_access_token', $access_token['oauth_token']);
      variable_set('qbo_api_access_secret', $access_token['oauth_token_secret']);
      variable_set('qbo_api_auth_date', REQUEST_TIME);
      variable_set('qbo_api_company_id', $_REQUEST['realmId']);
      unset($_SESSION['qbo_api_secret']);

      // write JS to pup up to refresh parent and close popup.
      echo '<script type="text/javascript">
              window.opener.location.href = window.opener.location.href;
              window.close();
            </script>';
    }
  }
  catch (OAuthException $e) {
    watchdog('qbo_api', $e->getMessage(), array(), WATCHDOG_ERROR);
    return '';
  }
}
